#region Using Directives

using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using System.ComponentModel;

#endregion

namespace Web.YUI
{
    /// <summary>
    /// The Overlay control is an extension of Module; its role is to facilitate the creation of modular content that is absolutely positioned above the flow of a page. It adds additional functionality to Module, including methods for positioning, multiple custom events for monitoring internal property changes, and a built-in IFRAME solution for dealing with SELECT element bleed-through in Internet Explorer.
    /// </summary>
    [ParseChildren(true), PersistChildren(false)]
    public abstract class Overlay : Module
    {
        #region Fields

        private AnimationCollection animations = null;

        #endregion

        #region Properties

        /// <summary>
        /// If set to true the Menu will have and iframe behind it to prevent other elements with a higher z-index from poking through. 
        /// </summary>
        public bool UseIFrame
        {
            get
            {
                object o = ViewState["UseIFrame"];
                return o == null ? false : (bool)o;
            }
            set
            {
                ViewState["UseIFrame"] = value;
            }
        }

        /// <summary>
        /// If set to true the Overlay will try to remain inside the confines of the size of viewport.
        /// </summary>
        public bool ConstrainToViewport
        {
            get
            {
                object o = ViewState["ConstrainToViewport"];
                return o == null ? false : (bool)o;
            }
            set
            {
                ViewState["ConstrainToViewport"] = value;
            }
        }

        /// <summary>
        /// Specifies whether the Overlay should be automatically centered in the viewport on window scroll and resize.
        /// </summary>
        public bool FixedCenter
        {
            get
            {
                object o = ViewState["FixedCenter"];
                return o == null ? false : (bool)o;
            }
            set
            {
                ViewState["FixedCenter"] = value;
            }
        }

        /// <summary>
        /// Sets the element's "z-index" style property.
        /// </summary>
        public int? ZIndex
        {
            get
            {
                object o = ViewState["ZIndex"];
                return o == null ? new Nullable<int>() : ((int?)o).Value;
            }
            set
            {
                ViewState["ZIndex"] = value;
            }
        }

        /// <summary>
        /// Sets the element's "left" style property. 
        /// </summary>
        public int? X
        {
            get
            {
                object o = ViewState["X"];
                return o == null ? new Nullable<int>() : ((int?)o).Value;
            }
            set
            {
                ViewState["X"] = value;
            }
        }

        /// <summary>
        /// Sets the element's "top" style property.
        /// </summary>
        public int? Y
        {
            get
            {
                object o = ViewState["Y"];
                return o == null ? new Nullable<int>() : ((int?)o).Value;
            }
            set
            {
                ViewState["Y"] = value;
            }
        }

        [
            MergableProperty(false),
            PersistenceMode(PersistenceMode.InnerProperty),
        ]
        public AnimationCollection Animations
        {
            get
            {
                if (this.animations == null)
                {
                    this.animations = new AnimationCollection();
                    if (base.IsTrackingViewState)
                    {
                        this.animations.TrackViewState();
                    }
                }
                return this.animations;
            }
        }

        #endregion

        #region Build Options

        protected override DelimitedList<string> BuildOptions()
        {
            DelimitedList<string> options = base.BuildOptions();

            if (!Width.IsEmpty)
            {
                options.Add(String.Format("width: \"{0}\"", Width));
            }
            if (!Height.IsEmpty)
            {
                options.Add(String.Format("height: \"{0}\"", Height));
            }
            if (X.HasValue)
                options.Add(String.Format("x: {0}", X.Value));
            if (Y.HasValue)
                options.Add(String.Format("y: {0}", Y.Value));
            if (ZIndex.HasValue)
                options.Add(String.Format("zindex: {0}", ZIndex.Value));
            string animations = GetAnimationInit();
            if (!String.IsNullOrEmpty(animations))
                options.Add(animations);
            if(FixedCenter)
                options.Add("fixedcenter: true");
            if(ConstrainToViewport)
                options.Add("constraintoviewport: true");
            options.Add(String.Format("iframe : {0}", UseIFrame.ToString().ToLower()));
            return options;
        }

        private string GetAnimationInit()
        {
            if (animations == null || animations.Count == 0)
                return String.Empty;
            DelimitedList<string> l = new DelimitedList<string>(", ");
            foreach (Animation a in Animations)
            {
                StringBuilder s = new StringBuilder();
                s.Append("{");
                s.AppendFormat("effect:{0},duration:{1}", EnumHelper.GetEnumDescription(a.Type), a.Duration);
                s.Append("}");
                l.Add(s.ToString());
            }
            if (l.Count > 1)
                return String.Format("effect:[{0}]", l);
            return String.Format("effect: {0}", l);
        }

        #endregion
    }
}