#region Using Directives

using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using System.ComponentModel;

#endregion

namespace Web.YUI
{
    [NonVisualControl,ParseChildren(true), PersistChildren(false)]
    public class YahooScriptManager : CompositeControl
    {
        #region Fields

        private static readonly string ScriptServer = "http://yui.yahooapis.com/";
        private static readonly string DefaultScriptVersion = "2.2.2";
        private Dictionary<YahooScript, ScriptBlock> yahooScripts = new Dictionary<YahooScript, ScriptBlock>();
        private Dictionary<YahooStyleSheet, StyleBlock> yahooStyles = new Dictionary<YahooStyleSheet, StyleBlock>();
        private List<IYahooControl> yahooControls = new List<IYahooControl>();
        private ScriptBlock initScript;
        private ScriptCollection scripts = null;
        private Dictionary<Script, ScriptBlock> scriptBlocks = new Dictionary<Script, ScriptBlock>();

        #endregion

        #region Overriden Methods

        protected override void CreateChildControls()
        {
            base.CreateChildControls();
            CheckConfiguration();

            yahooControls = FindControls();
            foreach (IYahooControl c in yahooControls)
            {
                if (c.ScriptDependencies != null)
                {
                    foreach (YahooScript script in c.ScriptDependencies)
                    {
                        if (!yahooScripts.ContainsKey(script))
                        {
                            ScriptBlock block = GetScriptBlock(script);
                            yahooScripts[script] = block;
                            Controls.Add(block);
                        }
                    }
                }

                if (c.StyleDependencies != null)
                {
                    foreach (YahooStyleSheet style in c.StyleDependencies)
                    {
                        if (!yahooStyles.ContainsKey(style))
                        {
                            StyleBlock block = GetStyleBlock(style);
                            yahooStyles[style] = block;
                            Controls.Add(block);
                        }
                    }
                }

                foreach (Script script in Scripts)
                {
                    ScriptBlock block = new ScriptBlock();
                    block.IncludePath = script.Path;
                    scriptBlocks[script] = block;
                    Controls.Add(block);
                }
            }

            initScript = new ScriptBlock();
            initScript.ID = String.Format("{0}_initScript", ID);
            Controls.Add(initScript);
        }

        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);

            StringBuilder sb = new StringBuilder();
            foreach (IYahooControl control in yahooControls)
            {
                if (control is WebControl)
                {
                    sb.Append("\t");
                    sb.AppendFormat("var {0};", ((WebControl)control).ID);
                    sb.Append("\n");
                }
            }
            sb.Append("\tfunction init() {\n");

            foreach (IYahooControl control in yahooControls)
            {
                if (!String.IsNullOrEmpty(control.LoadScript))
                {
                    sb.Append("\t\t");
                    sb.Append(control.LoadScript);
                    sb.Append("\n");
                }
            }
            
            sb.Append("\t}\n\tYAHOO.util.Event.addListener(window, \"load\", init);");

            initScript.Contents = sb.ToString();
            initScript.Visible = yahooControls.Count > 0;
        }

        protected override void Render(HtmlTextWriter writer)
        {
            foreach (ScriptBlock script in yahooScripts.Values)
            {
                script.RenderControl(writer);
            }

            foreach (StyleBlock style in yahooStyles.Values)
            {
                style.RenderControl(writer);
            }

            foreach (ScriptBlock script in scriptBlocks.Values)
            {
                script.RenderControl(writer);
            }

            initScript.RenderControl(writer);
        }

        #endregion

        #region Methods

        private void CheckConfiguration()
        {
            if (HostLocation == HostLocation.Yahoo &&
                (ScriptType == ScriptType.Debug ||
                ScriptType == ScriptType.Standard))
            {
                throw new ConfigurationException("Yahoo hosted scripts only support Minified ScriptType");
            }
        }

        private string GetStylePath(string path)
        {
            if (HostLocation == HostLocation.Yahoo)
            {
                return String.Format("{0}{1}/build/{2}", ScriptServer, ScriptVersion, path);
            }
            return String.Format("{0}build/{1}", LocalBasePath.EndsWith("/") ? LocalBasePath : LocalBasePath + "/", path);
        }

        private string GetScriptPath(string path, ScriptType availableTypes)
        {
            if (ScriptType == ScriptType.Minified && ((availableTypes & ScriptType.Minified) == ScriptType.Minified))
            {
                path = String.Format("{0}-min.js", path);
            }
            else if (ScriptType == ScriptType.Debug && ((availableTypes & ScriptType.Debug) == ScriptType.Debug))
            {
                path = String.Format("{0}-debug.js", path);
            }
            else
            {
                path = String.Format("{0}.js", path);
            }

            if (HostLocation == HostLocation.Yahoo)
            {
                return String.Format("{0}{1}/build/{2}", ScriptServer, ScriptVersion, path);
            }
            return String.Format("{0}build/{1}", LocalBasePath.EndsWith("/") ? LocalBasePath : LocalBasePath + "/", path);
        }

        private ScriptBlock GetScriptBlock(YahooScript script)
        {
            ScriptBlock block = new ScriptBlock();
            FieldInfo fi = script.GetType().GetField(script.ToString());
            YahooAssetAttribute ya = CheckYahooAsset(fi);
            if (ya != null)
            {
                block.IncludePath = GetScriptPath(ya.Path, ya.AvailableTypes);
                return block;
            }
            else
            {
                ResourceAssetAttribute ra = CheckResourceAsset(fi);
                if (ra != null)
                {
                    block.ResourceAssembly = ra.Assembly;
                    block.ResourceName = ra.Resource;
                    return block;
                }
            }
            throw new ArgumentException(script.ToString() + " does not have any script attributes");
        }

        private StyleBlock GetStyleBlock(YahooStyleSheet stylesheet)
        {
            StyleBlock block = new StyleBlock();
            FieldInfo fi = stylesheet.GetType().GetField(stylesheet.ToString());
            YahooAssetAttribute ya = CheckYahooAsset(fi);
            if (ya != null)
            {
                block.IncludePath = GetStylePath(ya.Path);
                return block;
            }
            else
            {
                ResourceAssetAttribute ra = CheckResourceAsset(fi);
                if (ra != null)
                {
                    block.ResourceAssembly = ra.Assembly;
                    block.ResourceName = ra.Resource;
                    return block;
                }
            }
            throw new ArgumentException(stylesheet.ToString() + " does not have any script attributes");
        }

        private ResourceAssetAttribute CheckResourceAsset(FieldInfo field)
        {
            ResourceAssetAttribute[] attributes = (ResourceAssetAttribute[])field.GetCustomAttributes(typeof(ResourceAssetAttribute), false);
            if (attributes != null && attributes.Length > 0)
            {
                return attributes[0];
            }
            return null;
        }

        private YahooAssetAttribute CheckYahooAsset(FieldInfo field)
        {
            YahooAssetAttribute[] attributes = (YahooAssetAttribute[])field.GetCustomAttributes(typeof(YahooAssetAttribute), false);
            if (attributes != null && attributes.Length > 0)
            {
                return attributes[0];
            }
            return null;
        }

        private List<IYahooControl> FindControls()
        {
            List<IYahooControl> foundControls = new List<IYahooControl>();
            ControlHelper.RecurseControls(Page.Controls, delegate(Control c)
            {
                if (c is IYahooControl)
                    foundControls.Add((IYahooControl)c);
            });
            return foundControls;
        }

        #endregion

        #region Properties

        [
            MergableProperty(false),
            PersistenceMode(PersistenceMode.InnerProperty),
        ]
        public new ScriptCollection Scripts
        {
            get
            {
                if (this.scripts == null)
                {
                    this.scripts = new ScriptCollection();
                    if (base.IsTrackingViewState)
                    {
                        this.scripts.TrackViewState();
                    }
                }
                return this.scripts;
            }
        }

        public string LocalBasePath
        {
            get
            {
                object o = ViewState["LocalBasePath"];
                return o == null ? ConfigHelper.GetString("YahooScriptManager.LocalBasePath", "/yui/") : (string)o;
            }
            set
            {
                ViewState["LocalBasePath"] = value;
            }
        }

        public ScriptType ScriptType
        {
            get
            {
                object o = ViewState["ScriptType"];
                return o == null ? ConfigHelper.GetEnum<ScriptType>("YahooScriptManager.ScriptType", ScriptType.Minified) : (ScriptType)o;
            }
            set
            {
                ViewState["ScriptType"] = value;
            }
        }

        public string ScriptVersion
        {
            get
            {
                object o = ViewState["ScriptVersion"];
                return o == null ? ConfigHelper.GetString("YahooScriptManager.ScriptVersion", DefaultScriptVersion) : (string)o;
            }
            set
            {
                ViewState["ScriptVersion"] = value;
            }
        }

        public HostLocation HostLocation
        {
            get
            {
                object o = ViewState["HostLocation"];
                return o == null ? ConfigHelper.GetEnum<HostLocation>("YahooScriptManager.HostLocation", HostLocation.Yahoo) : (HostLocation)o;
            }
            set
            {
                ViewState["HostLocation"] = value;
            }
        }

        #endregion
    }
}